package com.syntevo.plugin.jira.commit.messagesource;

import java.util.*;
import javax.swing.*;

import com.syntevo.openapi.gui.*;
import com.syntevo.openapi.gui.dialog.*;
import com.syntevo.openapi.smartsvn.command.commit.*;
import com.syntevo.openapi.smartsvn.settings.*;
import com.syntevo.plugin.jira.commit.*;
import com.syntevo.plugin.jira.transport.*;

/**
 * @author syntevo GmbH
 */
final class JiraCommitMessageSource implements ICommitMessageSource, JiraIssueTable {

	// Fields =================================================================

	private final DialogDisplayer dialogDisplayer;
	private final GuiComponentFactory componentFactory;
	private final JiraIssueLoader issueLoader;
	private final JiraQueryConfiguration queryConfiguration;

	private List<JiraIssue> pendingIssues = new ArrayList<JiraIssue>();
	private JiraCommitMessageSourceDialog currentDialog;

	// Setup ==================================================================

	public JiraCommitMessageSource(String baseUrl, SettingsServices settingsServices, String projectKey, GuiComponentFactory componentFactory, GuiSpacings spacings, DialogDisplayer dialogDisplayer) {
		this.dialogDisplayer = dialogDisplayer;
		this.componentFactory = componentFactory;

		issueLoader = new JiraIssueLoader(this);
		issueLoader.start();

		final boolean loadAll = "true".equals(System.getProperty("smartsvn.plugin.jira.load-all-issues", "false"));
		this.queryConfiguration = new JiraQueryConfiguration(new JiraUIConnection(baseUrl, settingsServices, false, spacings, dialogDisplayer), projectKey, loadAll);

		final JiraQueryConfiguration silentQueryConfiguration = new JiraQueryConfiguration(new JiraUIConnection(baseUrl, settingsServices, true, spacings, dialogDisplayer), projectKey, loadAll);
		issueLoader.load(silentQueryConfiguration, true);
	}

	// Implemented ============================================================

	public String getMenuItemName() {
		return "Get from JIRA...";
	}

	public String getCommitMessage() {
		final JiraCommitMessageSourceDialog dialog = new JiraCommitMessageSourceDialog(pendingIssues, componentFactory, issueLoader, queryConfiguration);
		currentDialog = dialog;

		try {
			issueLoader.load(queryConfiguration, false);
			if (!dialogDisplayer.show(currentDialog)) {
				return null;
			}
		}
		finally {
			currentDialog = null;
		}

		return dialog.getCommitMessage();
	}

	public void cleanup() {
		issueLoader.abortAndCleanup();
	}

	public void setIssues(final List<JiraIssue> issues) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				pendingIssues = new ArrayList<JiraIssue>(issues);
				if (currentDialog != null) {
					currentDialog.setIssues(pendingIssues);
				}
			}
		});
	}
}